\name{pmSolver-methods}
\docType{methods}
\alias{pmSolver-methods}
\alias{pmSolver}
\alias{pmSolver,PCME,userConstraints-method}
\title{Methods: Partial Moment Quadratic Portfolio Solver}
\description{
Implements the quadratic portfolio optimization algorithm using the partial co-moments symmetric
expectations matrix
}
\usage{
pmSolver(pcmExpectation, userConstraints, npoints=200, type=c("I","II","III","IV"))
}
\arguments{
  \item{pcmExpectation}{The symmetric co-partial moments. An \code{\linkS4class{PCME}} object created by calling 
  the \code{\link{pcmExpectation}} method.}
  \item{userConstraints}{ The portfolio constraints object (\code{\linkS4class{userConstraints}}) created by 
  calling the \code{\link{constraints}} method. }
  \item{npoints}{ (Optional) No of frontier points.}
  \item{type}{ Optimization type required. Valid methods are \dQuote{I} for the Minimum Risk Portfolio,
  \dQuote{II} for the Minimizing risk for a given level of return, \dQuote{III} for maximizing the risk adjusted 
  return for a given risk aversion coefficient, and \dQuote{IV} for the efficient frontier.}
}
\value{
An object of class \code{\linkS4class{PMSOLVER}}.
}
\examples{
\dontrun{
data(djiaret)
# calculate sample lower partial moments by bootstrap
# REM: for optimization you should use standardized moments 
# (standardize=TRUE)
sample.lpm<-pmExpectation(data=as.matrix(djiaret), method=rep("boot",30), 
			threshold=-0.01, moment=2, type="lower", standardize=TRUE)
# calculate sample lower partial co-moment matrix
sample.spcm<-pcmExpectation(sample.lpm, corr=cor(as.matrix(djiaret)), tail="lower")
##############################################
# Lower Partial Moments Portfolio Optimization
##############################################
# Test13: Constraints Setup
cst<-constraints(forecasts=apply(as.matrix(djiaret),2,FUN=function(x) mean(x)), 
		portfolioReturn=0.1/252, riskAversion = 2, riskFree=0.05/252, 
		budget = 1, assetsUB = rep(0.25,30),assetsLB=rep(0,30))

# Test14: Minimum Risk Solver (I)
port3<-pmSolver(sample.spcm, cst, type="I")
show(port3)
weightsPie(port3)

# Test15: Minimize Risk for given level of Return (II)
# REM portfolioReturn=0.1/252 (in constraints)
port4<-pmSolver(sample.spcm, cst, type="II")
show(port4)
weightsPie(port4)

# Test16: Maximize the risk adjusted return for a given risk aversion 
# coefficient (III)
getriskAversion(cst)
setriskAversion(cst) = 0.1
port5<-pmSolver(sample.spcm, cst, type="III")
show(port5)
weightsPie(port5)

getriskAversion(cst)
setriskAversion(cst) = 40
port6<-pmSolver(sample.spcm, cst, type="III")
show(port6)
weightsPie(port6)

# Test17: Efficient Frontier (IV)
port7<-pmSolver(sample.spcm, cst, npoints=200, type="IV")

plot(port7)
dev.new(2)
weightsPlot(port7)
}
}
\keyword{methods}
