\name{pmUtility}
\docType{methods}
\alias{pmUtility}
\alias{pmUtility-methods}
\alias{pmUtility,ANY-method}
\alias{pmUtility,vector-method}
\title{Methods: Partial Moments Utility}
\description{
An interesting Utility function characterized by upper to lower partial moments.
}
\usage{
pmUtility(data, threshold, urc, lrc, lmoment = 1, umoment = 1)
}
\arguments{
  \item{data}{ A data vector of differences or returns for which the utility is to be calculated. }
  \item{threshold}{ The threshold value for dividing upper to lower utility. }
  \item{urc}{ Upper Partial Moment Utility Risk Aversion Coefficient. }
  \item{lrc}{ Lower Partial Moment Utility Risk Aversion Coefficient.  }
  \item{lmoment}{ Lower Moment (need not be integer, but must be positive). }
  \item{umoment}{ Upper Moment (need not be integer, but must be positive). }
}
\value{
An object of class \code{\linkS4class{PMU}}
}
\references{
Farinelli, S. and Tibiletti, L. , \emph{Sharpe thinking in asset ranking with one-sided measures}, European Journal of Operational Research
Volume 185, Issue 3, 16 March 2008, Pages 1542-1547.\cr
}
\author{Alexios Ghalanos}
\note{
Has methods for show, plot and lines.\cr
This is an interesting utility allowing for the representation of the well known S-shaped utilty discussed in Kahnemann and Tversky's
prospect theory. Alternatively, and depending on the coefficients chosen, an inverse S-shaped utility can be constructed.
Portfolio optimization with respect to this utility function is non-trivial as it is non-quasi concave, and genetic optimization methods
have been suggested.
}
\examples{
\dontrun{
data(djiaret)
# Test18: Partial Moments Utility Test and Plots (S-Shaped)
putil1<-pmUtility(as.matrix(djiaret[,1]),threshold=0,urc=1,lrc=1,
lmoment=0.2,umoment=0.2)
show(putil1)
plot(putil1,col=2)
putil2<-pmUtility(as.matrix(djiaret[,1]),threshold=0,urc=1,lrc=1,lmoment=0.3,
umoment=0.3)
lines(putil2,col=3,text.col=3)
putil3<-pmUtility(as.matrix(djiaret[,1]),threshold=0,urc=1,lrc=1,lmoment=0.4,
umoment=0.4)
lines(putil3,col=4)
putil4<-pmUtility(as.matrix(djiaret[,1]),threshold=0,urc=1,lrc=1,lmoment=1.2,
umoment=0.4)
lines(putil4,col=5,text.col=5)
abline(v=0,col="grey")
leg.text<-c(paste("LM=",putil1@lmoment,", UM=",putil1@umoment,", LC=",putil1@lrc,", UC=",putil1@urc,sep=""),
paste("LM=",putil2@lmoment,", UM=",putil2@umoment,", LC=",putil2@lrc,", UC=",putil2@urc,sep=""),
paste("LM=",putil3@lmoment,", UM=",putil3@umoment,", LC=",putil3@lrc,", UC=",putil3@urc,sep=""),
paste("LM=",putil4@lmoment,", UM=",putil4@umoment,", LC=",putil4@lrc,", UC=",putil4@urc,sep=""))
legend("topleft",legend=leg.text,col=2:5,fill=2:5,cex=0.8)


# Test19: Partial Moments Utility Test and Plots (Inverse S-Shaped)
putil1<-pmUtility(as.matrix(djiaret[,1]),threshold=0,urc=1,lrc=1,lmoment=2.2, 
umoment=2.2)
show(putil1)
plot(putil1,col=2)
putil2<-pmUtility(as.matrix(djiaret[,1]),threshold=0,urc=1,lrc=1,lmoment=2.3,
umoment=2.3)
lines(putil2,col=3,text.col=3)
putil3<-pmUtility(as.matrix(djiaret[,1]),threshold=0,urc=1,lrc=1,lmoment=2.4,
umoment=2.4)
lines(putil3,col=4)
putil4<-pmUtility(as.matrix(djiaret[,1]),threshold=0,urc=1,lrc=1,lmoment=2.5,
umoment=2.5)
lines(putil4,col=5,text.col=5)
abline(v=0,col="grey")
leg.text<-c(paste("LM=",putil1@lmoment,", UM=",putil1@umoment,", LC=",putil1@lrc,", UC=",putil1@urc,sep=""),
paste("LM=",putil2@lmoment,", UM=",putil2@umoment,", LC=",putil2@lrc,", UC=",putil2@urc,sep=""),
paste("LM=",putil3@lmoment,", UM=",putil3@umoment,", LC=",putil3@lrc,", UC=",putil3@urc,sep=""),
paste("LM=",putil4@lmoment,", UM=",putil4@umoment,", LC=",putil4@lrc,", UC=",putil4@urc,sep=""))
legend("topleft",legend=leg.text,col=2:5,fill=2:5,cex=0.8)
}}
\keyword{methods}
